%{
/*
 *  ipfm.y - functions parsing the configuration file
 *  
 *  Robert Cheramy <tibob@via.ecp.fr>, 1999
 *
 *  Thanks to Etienne BERNARD <eb@via.ecp.fr> for his
 *  * little manual for Lex and Yacc
 *    http://www.via.ecp.fr/~eb/textes/minimanlexyacc-english.html
 *  * sample code in ippl (http://www.via.ecp.fr/~hugo/ippl)
 *  which helped me to understand Lex and Yacc
 */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <db.h>
#include <stdlib.h>
/* needed by F/BSD for <netinet/ip> */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in_systm.h>

#include <netinet/in.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include "filter.h"

void yyerror(char *);
void parseerror(char *, int);
int yylex(void);

extern int line;

extern struct s_filter *p_filter;
extern unsigned long int time_interval;
extern char *filemask;
extern char *device;
%}


%union {
  long            longval;
  char            *stringval;
  struct s_host   hostval;
  struct s_filter *filterval;
}


%token SLASH
%token LOG
%token IGNORE
%token EOL
%token TIME
%token FILENAME
%token DEVICE

%token<longval>   NUMBER
%token<longval>   DATETYPE
%token<stringval> IP
%token<stringval> STRING


%type<hostval>   Domain
%type<longval>   Action
%type<longval>   Time
%type<longval>   Date
%type<stringval> File
%type<stringval> Device
%type<filterval> Rule

%start Input
%%

Input:
          /* Empty */
        | Input Line
        ;

Line: EOL
| Rule EOL {
  $1->next = p_filter; p_filter = $1;
}

| Time EOL {
  time_interval = $1;
}

| File EOL {
  filemask = $1;
}

| Device EOL {
  device = $1;
}

| error EOL {
  parseerror("Skiping invalid line", line-1);
}

;

Rule:      Action Domain {
  $$       = (struct s_filter *)malloc(sizeof(struct s_filter));
  $$->log  = $1;
  $$->host = $2;
}
;


Action:    LOG       { $$ = 1; }
| IGNORE    { $$ = 0; }
;

Domain:	  IP {
  struct in_addr inp;
  inet_aton($1, &inp);
  $$.ip = inp.s_addr;
  $$.mask = 0xffffffff;
  free($1);
}

| IP SLASH IP {
  struct in_addr inp;
  inet_aton($1, &inp);
  $$.ip = inp.s_addr;
  inet_aton($3, &inp);
  $$.mask = inp.s_addr;
  free($1); free($3);
}
;

Time:   TIME Date {
  $$ = $2;
}
;

Date:   Date NUMBER DATETYPE {
  $$ = $1 + $2 * $3;
}
| NUMBER DATETYPE {
  $$ = $1 * $2;
}
;

File:  FILENAME STRING {
  $$ = $2;
}
;

Device: DEVICE STRING {
  $$ = $2;
}
;

%%

void yyerror(char *s) {
  printf("yacc error : %s\n",s);
}

void parseerror(char *s, int line) {
  printf("Parse error line %d: %s\n", line, s);
}
