/*
 *  filter.c - Filtering mechanism
 *  
 *  Robert Cheramy <tibob@via.ecp.fr>, 1999
 */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <db.h>
/* needed by F/BSD for <netinet/ip> */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <unistd.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <pcap.h>
#include "filter.h"
#include "utils.h"
#include "data.h"

unsigned long int time_interval = 0;
unsigned long int next_time;
struct s_filter *p_filter = NULL;
char *filemask = NULL;
int yyparse(void);
extern int line;
extern char * device;

/* look if the packet should be selected and handles it */
void dofilter (struct ip *p_packet) {
  struct s_filter *p_filtertemp;
  
  /* Look if IP source address is in the filter list */
  for(p_filtertemp = p_filter;
      p_filtertemp != NULL;
      p_filtertemp = p_filtertemp->next) {
    if (isonsubnet(p_packet->ip_src.s_addr, p_filtertemp->host.ip , p_filtertemp->host.mask)) {
      if (p_filtertemp->log) {
	data_add(p_packet->ip_src.s_addr, 0, ntohs(p_packet->ip_len));
      }
      break;
    }
  }
  
  /* Look if IP destination address is in the filter list */
  for(p_filtertemp = p_filter;
      p_filtertemp != NULL;
      p_filtertemp = p_filtertemp->next) {
    if (isonsubnet(p_packet->ip_dst.s_addr, p_filtertemp->host.ip , p_filtertemp->host.mask)) {
      if (p_filtertemp->log) {
	data_add(p_packet->ip_dst.s_addr, ntohs(p_packet->ip_len), 0);
      }
      break;
    }
  }
}

void configure(char *configfile) {
  FILE *p_configfile;
  int fd;
  //  int fd0;

  p_configfile = fopen(configfile,"r");
  if (p_configfile == NULL) {
    printf("Cannot open configuration file %s.\n", configfile);
    exit(1);
  }
  
  fd = fileno(p_configfile);
  //  fd0 = dup(0);
  dup2(fd, 0);

  line = 1;

  yyparse();

  fclose(p_configfile);
  //  dup2(fd0, 0);

  if (time_interval == 0) {
    printf("No time interval specified. Using 1 day as default.\n");
    time_interval = 24*60*60;
  }
  /* Set the time of the next dump */
  next_time = time(NULL) + time_interval;

  if (filemask == NULL) {
    printf("no output file specified. Exiting.\n");
    exit;
  }

  if (device == NULL) {
    char errbuf[PCAP_ERRBUF_SIZE];
    char * tdev;

    tdev = pcap_lookupdev(errbuf);

    if (!tdev)
    {
      fprintf(stderr, "pcap_lookupdev error: %s\n", errbuf);
      exit(1);
    }

    device = strdup(tdev);
  }
}
